from collections import OrderedDict
from typing import Optional, Sequence, Set, Union


class Structure:
    def __init__(
        self,
        name: str,
        desc: str,
        pattern: str = "",
        matches: Optional[list] = None,
        requirements: Optional[list] = None,
    ) -> None:
        """
        :param name: name of the structure
        :param desc: description of the structure
        :param pattern: Tregex pattern
        :param matches: matched subtrees by Tregex
        :param requirements: a list of structure names that this instance of Structure requires, e.g., MLS requires ["W","S"]
        """
        self.name = name
        self.desc = desc
        self.pattern = pattern
        if matches is None:
            self.matches = []
        else:
            self.matches = matches
        if requirements is None:
            self.requirements = []
        else:
            self.requirements = requirements
        self.freq: Union[float, int] = 0

    def __repr__(self) -> str:
        return (
            f"name: {self.name} ({self.desc})\nrequirements: {self.requirements}\npattern:"
            f" {self.pattern}\nmatches: {self.matches}\nfrequency: {self.freq}"
        )

    def __truediv__(self, other) -> Union[float, int]:
        return round(self.freq / other.freq, 4) if other.freq else 0


class StructureCounter:
    def __init__(self, ifile="", selected_measures: Optional[Set[str]] = None) -> None:
        self.ifile = ifile
        if selected_measures is None:
            self.selected_measures = set()
        else:
            self.selected_measures = selected_measures
        self.W = Structure("W", "words")
        self.S = Structure("S", "sentences", "ROOT")
        self.VP1 = Structure("VP1", "regular verb phrases", "VP > S|SINV|SQ")
        self.VP2 = Structure(
            "VP2",
            "verb phrases in inverted yes/no questions or in wh-questions",
            "MD|VBZ|VBP|VBD > (SQ !< VP)",
        )
        self.C1 = Structure(
            "C1",
            "regular clauses",
            "S|SINV|SQ "
            "[> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
            "(VP [<# MD|VBP|VBZ|VBD | < CC < (VP <# MD|VBP|VBZ|VBD)])]",
        )
        self.C2 = Structure(
            "C2",
            "fragment clauses",
            "FRAG > ROOT !<< "
            "(S|SINV|SQ [> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
            "(VP [<# MD|VBP|VBZ|VBD | < CC < "
            "(VP <# MD|VBP|VBZ|VBD)])])",
        )
        self.T1 = Structure(
            "T1",
            "regular T-units",
            "S|SBARQ|SINV|SQ > ROOT | [$-- S|SBARQ|SINV|SQ !>> SBAR|VP]",
        )
        self.T2 = Structure(
            "T2",
            "fragment T-units",
            "FRAG > ROOT !<< (S|SBARQ|SINV|SQ > ROOT | [$-- S|SBARQ|SINV|SQ !>> SBAR|VP])",
        )
        self.CN1 = Structure(
            "CN1",
            "complex nominals, type 1",
            "NP !> NP [<< JJ|POS|PP|S|VBG | << (NP $++ NP !$+ CC)]",
        )
        self.CN2 = Structure(
            "CN2",
            "complex nominals, type 2",
            "SBAR [<# WHNP | <# (IN < That|that|For|for) | <, S] & [$+ VP | > VP]",
        )
        self.CN3 = Structure("CN3", "complex nominals, type 3", "S < (VP <# VBG|TO) $+ VP")
        self.DC = Structure(
            "DC",
            "dependent clauses",
            "SBAR < "
            "(S|SINV|SQ "
            "[> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
            "(VP [<# MD|VBP|VBZ|VBD | < CC < (VP <# MD|VBP|VBZ|VBD)])])",
        )
        self.CT = Structure(
            "CT",
            "complex T-units",
            "S|SBARQ|SINV|SQ [> ROOT | [$-- S|SBARQ|SINV|SQ !>> SBAR|VP]] << "
            "(SBAR < (S|SINV|SQ "
            "[> ROOT <, (VP <# VB) | <# MD|VBZ|VBP|VBD | < "
            "(VP [<# MD|VBP|VBZ|VBD | < CC < "
            "(VP <# MD|VBP|VBZ|VBD)])]))",
        )
        self.CP = Structure("CP", "coordinate phrases", "ADJP|ADVP|NP|VP < CC")

        self.VP = Structure("VP", "verb phrases", requirements=["VP1", "VP2"])
        self.C = Structure("C", "clauses", requirements=["C1", "C2"])
        self.T = Structure("T", "T-units", requirements=["T1", "T2"])
        self.CN = Structure("CN", "complex nominals", requirements=["CN1", "CN2", "CN3"])

        self.MLS = Structure("MLS", "mean length of sentence", requirements=["W", "S"])
        self.MLT = Structure("MLT", "mean length of T-unit", requirements=["W", "T"])
        self.MLC = Structure("MLC", "mean length of clause", requirements=["W", "C1"])
        self.C_S = Structure("C_S", "clauses per sentence", requirements=["C1", "S"])
        self.VP_T = Structure("VP_T", "verb phrases per T-unit", requirements=["VP1", "T"])
        self.C_T = Structure("C_T", "clauses per T-unit", requirements=["C1", "T"])
        self.DC_C = Structure("DC_C", "dependent clauses per clause", requirements=["DC", "C1"])
        self.DC_T = Structure("DC_T", "dependent clauses per T-unit", requirements=["DC", "T"])
        self.T_S = Structure("T_S", "T-units per sentence", requirements=["T", "S"])
        self.CT_T = Structure("CT_T", "complex T-unit ratio", requirements=["CT", "T"])
        self.CP_T = Structure("CP_T", "coordinate phrases per T-unit", requirements=["CP", "T"])
        self.CP_C = Structure("CP_C", "coordinate phrases per clause", requirements=["CP", "C1"])
        self.CN_T = Structure("CN_T", "complex nominals per T-unit", requirements=["CN", "T"])
        self.CN_C = Structure("CN_C", "complex nominals per clause", requirements=["CN", "C1"])

        self.structures_to_query: Sequence[Structure] = (
            self.W,
            self.S,
            self.VP1,
            self.VP2,
            self.C1,
            self.C2,
            self.T1,
            self.T2,
            self.CN1,
            self.CN2,
            self.CN3,
            self.DC,
            self.CT,
            self.CP,
        )
        self.structures_to_report: Sequence[Structure] = (
            self.W,
            self.S,
            self.VP,
            self.C,
            self.T,
            self.DC,
            self.CT,
            self.CP,
            self.CN,
            self.MLS,
            self.MLT,
            self.MLC,
            self.C_S,
            self.VP_T,
            self.C_T,
            self.DC_C,
            self.DC_T,
            self.T_S,
            self.CT_T,
            self.CP_T,
            self.CP_C,
            self.CN_T,
            self.CN_C,
        )

        self.parse_selected_measures()
        self.fields = "Filename," + ",".join(
            (structure.name for structure in self.structures_to_report)
        ).replace("_", "/")

    def parse_selected_measures(self):
        if len(self.selected_measures) > 0:
            self.structures_to_report = [
                structure
                for structure in self.structures_to_report
                if structure.name in self.selected_measures
            ]
            selected_measures_extended = self.selected_measures.copy()
            for structure in self.structures_to_report:
                for name in structure.requirements:
                    if name not in self.selected_measures:
                        selected_measures_extended.add(name)
            self.structures_to_query = [
                structure
                for structure in self.structures_to_query
                if structure.name in selected_measures_extended
            ]

    def update_freqs(self) -> None:
        """
        Update frequencies of complex nominals, clauses, verb phrases, and T-units
        """
        self.VP.freq = self.VP1.freq + self.VP2.freq
        self.C.freq = self.C1.freq + self.C2.freq
        self.T.freq = self.T1.freq + self.T2.freq
        self.CN.freq = self.CN1.freq + self.CN2.freq + self.CN3.freq

    def compute_14_indicies(self) -> None:
        """
        Compute the 14 syntactic complexity indices
        """
        self.MLS.freq = self.W / self.S
        self.MLT.freq = self.W / self.T
        self.MLC.freq = self.W / self.C1
        self.C_S.freq = self.C1 / self.S
        self.VP_T.freq = self.VP1 / self.T
        self.C_T.freq = self.C1 / self.T
        self.DC_C.freq = self.DC / self.C1
        self.DC_T.freq = self.DC / self.T
        self.T_S.freq = self.T / self.S
        self.CT_T.freq = self.CT / self.T
        self.CP_T.freq = self.CP / self.T
        self.CP_C.freq = self.CP / self.C1
        self.CN_T.freq = self.CN / self.T
        self.CN_C.freq = self.CN / self.C1

    def get_freqs(self) -> dict:
        self.update_freqs()
        self.compute_14_indicies()
        freq_dict = OrderedDict({"Filename": self.ifile})
        for structure in self.structures_to_report:
            freq_dict[structure.name] = str(structure.freq)
        return freq_dict

    def __add__(self, other: "StructureCounter") -> "StructureCounter":
        new_ifile = self.ifile + "+" + other.ifile if self.ifile else other.ifile
        selected_measures = self.selected_measures | other.selected_measures
        new = StructureCounter(new_ifile, selected_measures=selected_measures)
        for structure in self.structures_to_query:
            exec(
                f"new.{structure.name}.freq = self.{structure.name}.freq +"
                f" other.{structure.name}.freq"
            )
        return new



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

import os
import re
from typing import Tuple

import jpype
from jpype import JClass

from .structure_counter import StructureCounter


class StanfordTregex:
    TREGEX_METHOD = "edu.stanford.nlp.trees.tregex.TregexPattern"
    MAX_MEMORY = "3072m"

    def __init__(
        self,
        dir_stanford_tregex: str = "",
        max_memory: str = MAX_MEMORY,
    ) -> None:
        self.classpath = os.path.join(dir_stanford_tregex, "stanford-tregex.jar")
        self.max_memory = max_memory
        self.init_tregex()

    def init_tregex(self):
        if not jpype.isJVMStarted():
            # Note that isJVMStarted may be renamed to isJVMRunning in the future.
            # In jpype's _core.py:
            # > TODO This method is horribly named.  It should be named isJVMRunning as
            # > isJVMStarted would seem to imply that the JVM was started at some
            # > point without regard to whether it has been shutdown.
            jpype.startJVM(f"-Xmx{self.max_memory}", classpath=self.classpath)
        else:
            jpype.addClassPath(self.classpath)

        self.TregexPattern = jpype.JClass("edu.stanford.nlp.trees.tregex.TregexPattern")
        self.StringReader = JClass("java.io.StringReader")
        self.PennTreeReader = JClass("edu.stanford.nlp.trees.PennTreeReader")
        self.patname_patobj = {}

    def query_pattern(self, patname: str, pattern: str, trees: str) -> Tuple[int, list]:
        matched_subtrees = []
        if patname not in self.patname_patobj:
            tregex_pattern = self.TregexPattern.compile(pattern)
            self.patname_patobj[patname] = tregex_pattern
        else:
            tregex_pattern = self.patname_patobj[patname]

        treeReader = self.PennTreeReader(self.StringReader(trees))
        tree = treeReader.readTree()
        while tree is not None:
            matcher = tregex_pattern.matcher(tree)
            last_matching_root_node = None
            while matcher.find():
                match = matcher.getMatch()
                if last_matching_root_node is not None and last_matching_root_node == match:
                    # implement Tregex's -o option: https://github.com/stanfordnlp/CoreNLP/blob/efc66a9cf49fecba219dfaa4025315ad966285cc/src/edu/stanford/nlp/trees/tregex/TregexPattern.java#L885
                    continue
                last_matching_root_node = match
                span_string = " ".join(str(leaf.toString()) for leaf in match.getLeaves())
                # we don't use match.spanString() because the output lacks whitespace, e.g., "the media" becomes "themedia"
                penn_string = str(match.pennString().replaceAll("\r", ""))
                matched_subtrees.append(span_string + "\n" + penn_string)
            tree = treeReader.readTree()
        return len(matched_subtrees), matched_subtrees

    def query(
        self,
        counter: StructureCounter,
        trees: str,
        is_reserve_matched: bool = False,
        odir_matched: str = "",
    ):
        for structure in counter.structures_to_query:
            print(f'\t[Tregex] Querying "{structure.desc}"...')
            if structure.name == "W":
                structure.freq = len(re.findall(r"\([A-Z]+\$? [^()—–-]+\)", trees))
                continue
            structure.freq, structure.matches = self.query_pattern(
                structure.name, structure.pattern, trees
            )
        if is_reserve_matched:
            self.write_match_output(counter, odir_matched)
        return counter

    def write_match_output(self, counter: StructureCounter, odir_matched: str = "") -> None:
        """
        Save Tregex's match output

        :param structures: an instance of Structures
        """
        bn_input = os.path.basename(counter.ifile)
        bn_input_noext = os.path.splitext(bn_input)[0]
        subdir_match_output = os.path.join(odir_matched, bn_input_noext).strip()
        if not os.path.isdir(subdir_match_output):
            # if not (exists and is a directory)
            os.makedirs(subdir_match_output)
        for structure in counter.structures_to_query:
            if structure.matches:
                bn_match_output = (
                    bn_input_noext + "-" + structure.name.replace("/", "p") + ".matches"
                )
                fn_match_output = os.path.join(subdir_match_output, bn_match_output)
                with open(fn_match_output, "w", encoding="utf-8") as f:
                    for match in structure.matches:
                        f.write(match + "\n")



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

import sys
import os

IS_WINDOWS = sys.platform == "win32"
IS_DARWIN = sys.platform == "darwin"
IS_LINUX = sys.platform == "linux"
if IS_WINDOWS and os.environ.get("AppData") is not None:
    USER_SOFTWARE_DIR = os.environ.get("AppData")
else:
    user_home = os.path.expanduser("~")
    USER_SOFTWARE_DIR = os.path.join(user_home, ".local", "share")



import os
import sys
from typing import Dict, List, Optional, Set

from .parser import StanfordParser
from .querier import StanfordTregex
from .structure_counter import StructureCounter


class NeoSCA:
    def __init__(
        self,
        ofile_freq: str = "result.csv",
        oformat_freq: str = "csv",
        dir_stanford_parser: str = "",
        dir_stanford_tregex: str = "",
        odir_matched: str = "",
        newline_break: str = "never",
        max_length: Optional[int] = None,
        selected_measures: Optional[Set[str]] = None,
        is_reserve_parsed: bool = False,
        is_reserve_matched: bool = False,
        is_stdout: bool = False,
        is_skip_querying: bool = False,
        is_verbose: bool = True,
    ) -> None:
        self.ofile_freq = ofile_freq
        self.oformat_freq = oformat_freq
        self.dir_stanford_parser = dir_stanford_parser
        self.dir_stanford_tregex = dir_stanford_tregex
        self.odir_matched = odir_matched
        self.newline_break = newline_break
        self.max_length = max_length
        self.selected_measures = selected_measures
        self.is_reserve_parsed = is_reserve_parsed
        self.is_reserve_matched = is_reserve_matched
        self.is_stdout = is_stdout
        self.is_skip_querying = is_skip_querying
        self.is_verbose = is_verbose
        self.counter_lists: List[StructureCounter] = []

        self.parser = StanfordParser(
            self.dir_stanford_parser,
            is_verbose=self.is_verbose,
        )
        self.tregex = StanfordTregex(
            dir_stanford_tregex=self.dir_stanford_tregex,
        )

    def _is_skip_parsing(self, ofile_parsed: str, ifile: str) -> bool:
        """See whether a parsed file already exists"""
        is_skip_parsing = False
        is_exist = os.path.exists(ofile_parsed)
        if is_exist:
            is_not_empty = os.path.getsize(ofile_parsed) > 0
            is_parsed_newer_than_input = os.path.getmtime(ofile_parsed) > os.path.getmtime(ifile)
            if is_not_empty and is_parsed_newer_than_input:
                is_skip_parsing = True
        return is_skip_parsing

    def _read_file(self, filename: str) -> str:
        """Read a file (either an input file or a parsed file) and return the content"""
        with open(filename, "r", encoding="utf-8") as f:
            content = f.read()
        return content

    def query_against_trees(self, trees: str, counter: StructureCounter) -> StructureCounter:
        counter = self.tregex.query(
            counter,
            trees,
            is_reserve_matched=self.is_reserve_matched,
            odir_matched=self.odir_matched,
        )
        return counter

    def parse_text(self, text: str, ofile_parsed="cmdline_text.parsed") -> str:
        trees = self.parser.parse(text, self.max_length, self.newline_break)
        if self.is_reserve_parsed:
            with open(ofile_parsed, "w", encoding="utf-8") as f:
                f.write(trees)
        return trees

    def run_on_text(self, text: str, ifile: str = "cmdline_text") -> None:
        trees = self.parse_text(text)
        if not self.is_skip_querying:
            counter = StructureCounter(ifile, selected_measures=self.selected_measures)
            counter = self.query_against_trees(trees, counter)
            self.counter_lists.append(counter)
            self.write_freq_output()

    def parse_ifile(self, ifile: str) -> str:
        """Parse a single file"""
        ofile_parsed = os.path.splitext(ifile)[0] + ".parsed"
        is_skip_parsing = self._is_skip_parsing(ofile_parsed=ofile_parsed, ifile=ifile)
        if is_skip_parsing:
            print(
                f"\t[Parser] Parsing skipped: {ofile_parsed} already"
                f" exists, and is non-empty and newer than {ifile}."
            )
            return self._read_file(ofile_parsed)
        text = self._read_file(ifile)
        try:
            trees = self.parse_text(text, ofile_parsed)
        except KeyboardInterrupt:
            if os.path.exists(ofile_parsed):
                os.remove(ofile_parsed)
            sys.exit(1)
        else:
            return trees

    def parse_ifile_and_query(self, ifile: str) -> StructureCounter:
        trees = self.parse_ifile(ifile)
        counter = StructureCounter(ifile, selected_measures=self.selected_measures)
        return self.query_against_trees(trees, counter)

    def run_on_ifiles(self, ifiles, is_combine=False) -> None:
        total = len(ifiles)
        if not self.is_skip_querying:
            if is_combine:
                parent_counter = StructureCounter(selected_measures=self.selected_measures)
                for i, ifile in enumerate(ifiles):
                    print(f'[NeoSCA] Processing "{ifile}" ({i+1}/{total})...')
                    child_counter = self.parse_ifile_and_query(ifile)
                    parent_counter += child_counter
                self.counter_lists.append(parent_counter)
            else:
                for i, ifile in enumerate(ifiles):
                    print(f'[NeoSCA] Processing "{ifile}" ({i+1}/{total})...')
                    child_counter = self.parse_ifile_and_query(ifile)
                    self.counter_lists.append(child_counter)
            self.write_freq_output()
        else:
            for i, ifile in enumerate(ifiles):
                print(f'[NeoSCA] Processing "{ifile}" ({i+1}/{total})...')
                self.parse_ifile(ifile)

    def write_freq_output(self) -> None:
        if self.oformat_freq == "csv":
            freq_output = StructureCounter(selected_measures=self.selected_measures).fields
            for structures in self.counter_lists:
                freq_dict = structures.get_freqs()
                freq_output += "\n" + ",".join(str(freq) for freq in freq_dict.values())
        elif self.oformat_freq == "json":
            import json

            final_freq_dict: Dict[str, List[Dict]] = {"Files": []}
            for structures in self.counter_lists:
                freq_dict = structures.get_freqs()
                final_freq_dict["Files"].append(freq_dict)
            freq_output = json.dumps(final_freq_dict)
        else:
            print(f"Unexpected output format: {self.oformat_freq}")
            sys.exit(1)

        if not self.is_stdout:
            with open(self.ofile_freq, "w", encoding="utf-8") as f:
                f.write(freq_output)
        else:
            sys.stdout.write(freq_output)



from .base_tmpl import BaseTmpl
from unittest import mock
from neosca.main import SCAUI

cmdline_text = "This is a test."


class TestMain(BaseTmpl):
    ui = SCAUI()

    @mock.patch("sys.version_info")
    def test_check_python(self, mock_version_info):
        mock_version_info.major, mock_version_info.minor = (3, 6)
        sucess, _ = self.ui.check_python()
        self.assertFalse(sucess)

    def test_show_version(self) -> None:
        self.assertTrue(self.ui.show_version())

    def test_list_fields(self):
        self.assertTrue(self.ui.list_fields())



import re
from .cmdline_tmpl import CmdlineTmpl

cmdline_text = "This is a test."


class TestCommandLineBasic(CmdlineTmpl):
    def test_no_file(self):
        result = self.template(["python", "-m", "neosca"], expected_output_file=None)
        result_stdout = result.stdout.decode("utf8")
        self.assertIn("help", result_stdout)
        #  Check for a few more arguments to ensure we hit the intended argumentParser
        self.assertIn("--text", result_stdout)
        self.assertIn("--output-file", result_stdout)
        self.assertIn("--no-query", result_stdout)

    def test_help(self):
        """Test that all three options print the same help page"""
        result = self.template(["python", "-m", "neosca"], expected_output_file=None)
        result_h = self.template(["python", "-m", "neosca", "-h"], expected_output_file=None)
        result_help = self.template(
            ["python", "-m", "neosca", "--help"], expected_output_file=None
        )

        self.assertEqual(result_h.stdout.decode("utf-8"), result_help.stdout.decode("utf-8"))
        self.assertEqual(result.stdout.decode("utf-8"), result_h.stdout.decode("utf-8"))

    def test_parse_text(self):
        self.template(
            ["python", "-m", "neosca", "--text", f"'{cmdline_text}'", "--no-query"],
            expected_output_file=[
                "cmdline_text.parsed",
            ],
        )

    def test_parse_text_and_query(self):
        self.template(
            [
                "python",
                "-m",
                "neosca",
                "--text",
                f"'{cmdline_text}'",
                "--reserve-parsed",
                "--reserve-matched",
            ],
            expected_output_file=[
                "result.csv",
                "cmdline_text.parsed",
                "result_matches",
            ],
        )

    def test_parse_ifiles(self):
        self.template(
            ["python", "-m", "neosca", "sample.txt", "--no-query"],
            expected_output_file=["sample.parsed"],
        )

    def test_parse_ifiles_and_query(self):
        self.template(
            ["python", "-m", "neosca", "sample.txt", "--reserve-parsed", "--reserve-matched"],
            expected_output_file=[
                "result.csv",
                "sample.parsed",
                "result_matches",
            ],
        )

    def test_outputfile(self):
        self.template(
            [
                "python",
                "-m",
                "neosca",
                "sample.txt",
                "--output-format",
                "csv",
                "--reserve-parsed",
                "--reserve-matched",
            ],
            expected_output_file=["result.csv", "sample.parsed", "result_matches"],
        )
        self.template(
            [
                "python",
                "-m",
                "neosca",
                "sample.txt",
                "--output-format",
                "json",
                "--reserve-parsed",
                "--reserve-matched",
            ],
            expected_output_file=[
                "result.json",
                "sample.parsed",
                "result_matches",
            ],
        )
        self.template(
            [
                "python",
                "-m",
                "neosca",
                "sample.txt",
                "--output-file",
                "sample.csv",
                "--reserve-parsed",
                "--reserve-matched",
            ],
            expected_output_file=[
                "sample.csv",
                "sample.parsed",
                "sample_matches",
            ],
        )
        self.template(
            [
                "python",
                "-m",
                "neosca",
                "sample.txt",
                "--output-file",
                "sample.json",
                "--reserve-parsed",
                "--reserve-matched",
            ],
            expected_output_file=[
                "sample.json",
                "sample.parsed",
                "sample_matches",
            ],
        )

    def test_list_fields(self):
        result = self.template(
            ["python", "-m", "neosca", "--list"], text=None, expected_output_file=None
        )
        result_stdout = result.stdout.decode("utf-8")
        ncorrect_lines = len(re.findall(r"^[A-Z_]+: .*$", result_stdout, re.MULTILINE))
        self.assertEqual(result_stdout.count("\n"), 23)
        self.assertEqual(ncorrect_lines, 23)

    def test_show_version(self):
        result = self.template(
            ["python", "-m", "neosca", "--version"], text=None, expected_output_file=None
        )
        self.assertRegex(result.stdout.decode("utf-8").strip(), r"[^.]+\.[^.]+\.[^.]+")

    def test_invalid_file(self):
        self.template(
            ["python", "-m", "neosca", "no_such_file.txt"],
            success=False,
            expected_output_file=[],
        )



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

from neosca.querier import StanfordTregex
from neosca.structure_counter import StructureCounter

from .base_tmpl import BaseTmpl
from .base_tmpl import dir_stanford_tregex
from .base_tmpl import tree


class TestStanfordTregex(BaseTmpl):
    def setUp(self):
        self.counter = StructureCounter()
        self.tregex = StanfordTregex(dir_stanford_tregex)
        return super().setUp()

    def test_query(self):
        structures = self.tregex.query(self.counter, tree)
        structures.update_freqs()
        self.assertEqual(1, structures.S.freq)
        self.assertEqual(2, structures.VP.freq)
        self.assertEqual(1, structures.C.freq)
        self.assertEqual(1, structures.T.freq)
        self.assertEqual(0, structures.DC.freq)
        self.assertEqual(0, structures.CT.freq)
        self.assertEqual(0, structures.CP.freq)
        self.assertEqual(1, structures.CN.freq)



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

from .base_tmpl import BaseTmpl
from neosca.structure_counter import Structure
from neosca.structure_counter import StructureCounter


class TestStructure(BaseTmpl):
    def setUp(self):
        self.W = Structure("W", "words")
        self.S = Structure("S", "sentences")
        return super().setUp()

    def test_div(self):
        self.W.freq = 25
        self.S.freq = 2
        self.assertEqual(12.5, self.W / self.S)
        self.S.freq = 0
        self.assertEqual(0, self.W / self.S)


class TestStructureCounter(BaseTmpl):
    def setUp(self):
        self.selected_measures = {"VP", "T", "DC_C"}
        self.counter = StructureCounter(selected_measures=self.selected_measures)
        return super().setUp()

    def test_update_freqs(self):
        self.counter.VP1.freq, self.counter.VP2.freq = 7, 1
        self.counter.C1.freq, self.counter.C2.freq = 9, 1
        self.counter.T1.freq, self.counter.T2.freq = 11, 1
        self.counter.CN1.freq, self.counter.CN2.freq, self.counter.CN3.freq = 13, 1, 2

        self.counter.update_freqs()
        self.assertEqual(8, self.counter.VP.freq)
        self.assertEqual(10, self.counter.C.freq)
        self.assertEqual(12, self.counter.T.freq)
        self.assertEqual(16, self.counter.CN.freq)

    def test_compute_14_indices(self):
        (
            self.counter.W.freq,
            self.counter.S.freq,
            self.counter.VP1.freq,
            self.counter.C1.freq,
            self.counter.T.freq,
            self.counter.DC.freq,
            self.counter.CT.freq,
            self.counter.CP.freq,
            self.counter.CN.freq,
        ) = (1530, 54, 174, 149, 83, 61, 52, 26, 181)
        self.counter.compute_14_indicies()
        self.assertEqual(1530, self.counter.W.freq)
        self.assertEqual(54, self.counter.S.freq)
        self.assertEqual(174, self.counter.VP1.freq)
        self.assertEqual(149, self.counter.C1.freq)
        self.assertEqual(83, self.counter.T.freq)
        self.assertEqual(61, self.counter.DC.freq)
        self.assertEqual(52, self.counter.CT.freq)
        self.assertEqual(26, self.counter.CP.freq)
        self.assertEqual(181, self.counter.CN.freq)
        self.assertEqual(28.3333, self.counter.MLS.freq)
        self.assertEqual(18.4337, self.counter.MLT.freq)
        self.assertEqual(10.2685, self.counter.MLC.freq)
        self.assertEqual(2.7593, self.counter.C_S.freq)
        self.assertEqual(2.0964, self.counter.VP_T.freq)
        self.assertEqual(1.7952, self.counter.C_T.freq)
        self.assertEqual(0.4094, self.counter.DC_C.freq)
        self.assertEqual(0.7349, self.counter.DC_T.freq)
        self.assertEqual(1.537, self.counter.T_S.freq)
        self.assertEqual(0.6265, self.counter.CT_T.freq)
        self.assertEqual(0.3133, self.counter.CP_T.freq)
        self.assertEqual(0.1745, self.counter.CP_C.freq)
        self.assertEqual(2.1807, self.counter.CN_T.freq)
        self.assertEqual(1.2148, self.counter.CN_C.freq)

    def test_get_freqs(self):
        freq_dict = self.counter.get_freqs()
        self.assertTrue("Filename", freq_dict.keys() - set(self.selected_measures))



#!/usr/bin/env python3
# -*- coding=utf-8 -*-

from .util_platform_info import IS_WINDOWS


class _bcolors:
    HEADER = "\033[95m"
    OKBLUE = "\033[94m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    BOLD = "\033[1m"
    UNDERLINE = "\033[4m"


bcolors = _bcolors()
color_support = True
if IS_WINDOWS:
    try:
        # https://stackoverflow.com/questions/36760127/...
        # how-to-use-the-new-support-for-ansi-escape-sequences-in-the-windows-10-console
        from ctypes import windll  # type:ignore

        kernel32 = windll.kernel32
        kernel32.SetConsoleMode(kernel32.GetStdHandle(-11), 7)
    except Exception:  # pragma: no cover
        color_support = False


def color_print(color: str, s: str, prefix: str = "", postfix: str = "", **kwargs) -> None:
    kwargs.update({"sep": ""})
    if color_support:
        print(prefix, bcolors.__getattribute__(color) + s + bcolors.ENDC, postfix, **kwargs)
    else:  # pragma: no cover
        print(prefix, s, postfix, **kwargs)


def same_line_print(s: str, width=80, **kwargs) -> None:
    print(f"\r{'':<{width}}", end="")  # clear the line
    print(f"\r{s}", end="", **kwargs)


def get_yes_or_no(prompt: str = "") -> str:
    prompt_options = "Enter [y]es or [n]o: "
    sep = "\n" if prompt else ""
    answer = input(prompt + sep + prompt_options)
    while answer not in ("y", "n", "Y", "N"):
        answer = input(f"Unexpected input: {answer}.\nEnter [y]es or [n]o: ")
    return answer
